@@ -30,7 +30,7 @@ package require Tcl 8.4
 package require fileutil ; # Required by importFile.
 package require uri
 
-package provide ncgi 1.4.3
+package provide ncgi 1.4.4
 
 namespace eval ::ncgi {
 
@@ -271,11 +271,11 @@ proc ::ncgi::decode {str} {
     set str [string map [list + { } "\\" "\\\\" \[ \\\[ \] \\\]] $str]
 
     # prepare to process all %-escapes
-    regsub -all -- {%([Ee][A-Fa-f0-9])%([89ABab][A-Fa-f0-9])%([89ABab][A-Fa-f0-9])} \
+    regsub -all -nocase -- {%([E][A-F0-9])%([89AB][A-F0-9])%([89AB][A-F0-9])} \
 	$str {[encoding convertfrom utf-8 [DecodeHex \1\2\3]]} str
-    regsub -all -- {%([CDcd][A-Fa-f0-9])%([89ABab][A-Fa-f0-9])}                     \
+    regsub -all -nocase -- {%([CDcd][A-F0-9])%([89AB][A-F0-9])} \
 	$str {[encoding convertfrom utf-8 [DecodeHex \1\2]]} str
-    regsub -all -- {%([0-7][A-Fa-f0-9])} $str {\\u00\1} str
+    regsub -all -nocase -- {%([A-F0-9][A-F0-9])} $str {\\u00\1} str
 
     # process \u unicode mapped chars
     return [subst -novar $str]
